@@ -11,15 +11,16 @@ import Common.Annex
 import Assistant
 import Command
 import Option
+import Utility.HumanTime
 
 def :: [Command]
-def = [withOptions [foregroundOption, stopOption] $ 
-	command "watch" paramNothing seek "watch for changes"]
+def = [notBareRepo $ withOptions [foregroundOption, stopOption] $ 
+	command "watch" paramNothing seek SectionCommon "watch for changes"]
 
 seek :: [CommandSeek]
 seek = [withFlag stopOption $ \stopdaemon -> 
 	withFlag foregroundOption $ \foreground ->
-	withNothing $ start False foreground stopdaemon]
+	withNothing $ start False foreground stopdaemon Nothing]
 
 foregroundOption :: Option
 foregroundOption = Option.flag [] "foreground" "do not daemonize"
@@ -27,9 +28,9 @@ foregroundOption = Option.flag [] "foreground" "do not daemonize"
 stopOption :: Option
 stopOption = Option.flag [] "stop" "stop daemon"
 
-start :: Bool -> Bool -> Bool -> CommandStart
-start assistant foreground stopdaemon = notBareRepo $ do
+start :: Bool -> Bool -> Bool -> Maybe Duration -> CommandStart
+start assistant foreground stopdaemon startdelay = do
 	if stopdaemon
 		then stopDaemon
-		else startDaemon assistant foreground Nothing -- does not return
+		else startDaemon assistant foreground startdelay Nothing Nothing -- does not return
 	stop
