@@ -81,7 +81,7 @@ gen r u c gc = do
 			, whereisKey = Nothing
 			, remoteFsck = Nothing
 			, repairRepo = Nothing
-			, config = M.empty
+			, config = c
 			, repo = r
 			, gitconfig = gc
 			, localpath = if islocal
